@@ -9,7 +9,7 @@ import enum
 import sys
 
 from decimal import Decimal
-from typing import Any, Dict, List, NamedTuple, Optional, Set, Tuple, Union
+from typing import Any, Dict, FrozenSet, List, NamedTuple, Optional, Set, Union
 
 from beancount.core.amount import Amount
 from beancount.core.number import D
@@ -40,6 +40,11 @@ class Booking(enum.Enum):
     # Reject ambiguous matches with an error.
     STRICT = 'STRICT'
 
+    # Strict booking method, but disambiguate further with sizes. Reject
+    # ambiguous matches with an error but if a lot matches the size exactly,
+    # accept it the oldest.
+    STRICT_WITH_SIZE = 'STRICT_WITH_SIZE'
+
     # Disable matching and accept the creation of mixed inventories.
     NONE = 'NONE'
 
@@ -55,22 +60,6 @@ class Booking(enum.Enum):
     # Highest-in first-out in the case of ambiguity.
     HIFO = 'HIFO'
 
-def new_directive(clsname, fields: List[Tuple]) -> NamedTuple:
-    """Create a directive class. Do not include default fields.
-    This should probably be carried out through inheritance.
-
-    Args:
-      name: A string, the capitalized name of the directive.
-      fields: A string or the list of strings, names for the fields
-        to add to the base tuple.
-    Returns:
-      A type object for the new directive type.
-    """
-    return NamedTuple(
-        clsname,
-        [('meta', Meta), ('date', datetime.date)] + fields)
-
-
 # All possible types of entries. These are the main data structures in use
 # within the program. They are all treated as immutable.
 #
@@ -84,285 +73,348 @@ def new_directive(clsname, fields: List[Tuple]) -> NamedTuple:
 #     shows up in the file is used as a secondary sort key beyond the date.
 
 
-# An "open account" directive.
-#
-# Attributes:
-#   meta: See above.
-#   date: See above.
-#   account: A string, the name of the account that is being opened.
-#   currencies: A list of strings, currencies that are allowed in this account.
-#     May be None, in which case it means that there are no restrictions on which
-#     currencies may be stored in this account.
-#   booking: A Booking enum, the booking method to use to disambiguate
-#     postings to this account (when zero or more than one postings match the
-#     specification), or None if not specified. In practice, this attribute will
-#     be should be left unspecified (None) in the vast majority of cases. See
-#     Booking below for a selection of valid methods.
-Open = new_directive('Open', [
-    ('account', Account),
-    ('currencies', List[Currency]),
-    ('booking', Booking)])
+class Open(NamedTuple):
+    """
+    An "open account" directive.
+
+    Attributes:
+      meta: See above.
+      date: See above.
+      account: A string, the name of the account that is being opened.
+      currencies: A list of strings, currencies that are allowed in this account.
+        May be None, in which case it means that there are no restrictions on which
+        currencies may be stored in this account.
+      booking: A Booking enum, the booking method to use to disambiguate
+        postings to this account (when zero or more than one postings match the
+        specification), or None if not specified. In practice, this attribute will
+        be should be left unspecified (None) in the vast majority of cases. See
+        Booking below for a selection of valid methods.
+    """
+    meta: Meta
+    date: datetime.date
+    account: Account
+    currencies: List[Currency]
+    booking: Optional[Booking]
 
 
-# A "close account" directive.
-#
-# Attributes:
-#   account: A string, the name of the account that is being closed.
-Close = new_directive('Close', [
-    ('account', Account)])
+class Close(NamedTuple):
+    """
+    A "close account" directive.
 
-# An optional commodity declaration directive. Commodities generally do not need
-# to be declared, but they may, and this is mainly created as intended to be
-# used to attach meta-data on a commodity name. Whenever a plugin needs
-# per-commodity meta-data, you would define such a commodity directive. Another
-# use is to define a commodity that isn't otherwise (yet) used anywhere in an
-# input file. (At the moment the date is meaningless but is specified for
-# coherence with all the other directives; if you can think of a good use case,
-# let us know).
-#
-# Attributes:
-#   meta: See above.
-#   date: See above.
-#   currency: A string, the commodity under consideration.
-Commodity = new_directive('Commodity', [
-    ('currency', Currency)])
-
-# A "pad this account with this other account" directive. This directive
-# automatically inserts transactions that will make the next chronological
-# balance directive succeeds. It can be used to fill in missing date ranges of
-# transactions, as a convenience. You don't have to use this, it's sugar coating
-# in case you need it, while you're entering past history into your Ledger.
-#
-# Attributes:
-#   meta: See above.
-#   date: See above.
-#   account: A string, the name of the account which needs to be filled.
-#   source_account: A string, the name of the account which is used to debit from
-#     in order to fill 'account'.
-Pad = new_directive('Pad', [
-    ('account', Account),
-    ('source_account', Account)])
-
-# A "check the balance of this account" directive. This directive asserts that
-# the declared account should have a known number of units of a particular
-# currency at the beginning of its date. This is essentially an assertion, and
-# corresponds to the final "Statement Balance" line of a real-world statement.
-# These assertions act as checkpoints to help ensure that you have entered your
-# transactions correctly.
-#
-# Attributes:
-#   meta: See above.
-#   date: See above.
-#   account: A string, the account whose balance to check at the given date.
-#   amount: An Amount, the number of units of the given currency you're
-#     expecting 'account' to have at this date.
-#   diff_amount: None if the balance check succeeds. This value is set to
-#     an Amount instance if the balance fails, the amount of the difference.
-#   tolerance: A Decimal object, the amount of tolerance to use in the
-#     verification.
-Balance = new_directive('Balance', [
-    ('account', Account),
-    ('amount', Amount),
-    ('tolerance', Optional[Decimal]),
-    ('diff_amount', Optional[Amount])])
+    Attributes:
+      meta: See above.
+      date: See above.
+      account: A string, the name of the account that is being closed.
+    """
+    meta: Meta
+    date: datetime.date
+    account: Account
 
 
-# Postings are contained in Transaction entries. These represent the individual
-# legs of a transaction. Note: a posting may only appear within a single entry
-# (multiple transactions may not share a Posting instance), and that's what the
-# entry field should be set to.
-#
-# Attributes:
-#   account: A string, the account that is modified by this posting.
-#   units: An Amount, the units of the position.
-#   cost: A Cost or CostSpec instances, the units of the position.
-#   price: An Amount, the price at which the position took place, or
-#     None, where not relevant. Providing a price member to a posting
-#     automatically adds a price in the prices database at the date of the
-#     transaction.
-#   flag: An optional flag, a one-character string or None, which is to be
-#     associated with the posting. Most postings don't have a flag, but it can
-#     be convenient to mark a particular posting as problematic or pending to
-#     be reconciled for a future import of its account.
-#   meta: A dict of strings to values, the metadata that was attached
-#     specifically to that posting, or None, if not provided. In practice, most
-#     of the instances will be unlikely to have metadata.
-Posting = NamedTuple('Posting', [
-    ('account', Account),
-    ('units', Amount),
-    ('cost', Optional[Union[Cost, CostSpec]]),
-    ('price', Optional[Amount]),
-    ('flag', Optional[Flag]),
-    ('meta', Optional[Meta])])
+class Commodity(NamedTuple):
+    """
+    An optional commodity declaration directive. Commodities generally do not need
+    to be declared, but they may, and this is mainly created as intended to be
+    used to attach meta-data on a commodity name. Whenever a plugin needs
+    per-commodity meta-data, you would define such a commodity directive. Another
+    use is to define a commodity that isn't otherwise (yet) used anywhere in an
+    input file. (At the moment the date is meaningless but is specified for
+    coherence with all the other directives; if you can think of a good use case,
+    let us know).
 
-# A transaction! This is the main type of object that we manipulate, and the
-# entire reason this whole project exists in the first place, because
-# representing these types of structures with a spreadsheet is difficult.
-#
-# Attributes:
-#   meta: See above.
-#   date: See above.
-#   flag: A single-character string or None. This user-specified string
-#     represents some custom/user-defined state of the transaction. You can use
-#     this for various purposes. Otherwise common, pre-defined flags are defined
-#     under beancount.core.flags, to flags transactions that are automatically
-#     generated.
-#   payee: A free-form string that identifies the payee, or None, if absent.
-#   narration: A free-form string that provides a description for the transaction.
-#     All transactions have at least a narration string, this is never None.
-#   tags: A set of tag strings (without the '#'), or EMPTY_SET.
-#   links: A set of link strings (without the '^'), or EMPTY_SET.
-#   postings: A list of Posting instances, the legs of this transaction. See the
-#     doc under Posting above.
-Transaction = new_directive('Transaction', [
-    ('flag', Flag),
-    ('payee', Optional[str]),
-    ('narration', str),
-    ('tags', Set),
-    ('links', Set),
-    ('postings', List[Posting])])
-
-# A pair of a Posting and its parent Transaction. This is inserted as
-# temporaries in lists of postings-of-entries, which is the product of a
-# realization.
-#
-# Attributes:
-#   txn: The parent Transaction instance.
-#   posting: The Posting instance.
-TxnPosting = NamedTuple('TxnPosting', [
-    ('txn', Transaction),
-    ('posting', Posting)])
+    Attributes:
+      meta: See above.
+      date: See above.
+      currency: A string, the commodity under consideration.
+    """
+    meta: Meta
+    date: datetime.date
+    currency: Currency
 
 
-# A note directive, a general note that is attached to an account. These are
-# used to attach text at a particular date in a specific account. The notes can
-# be anything; a typical use would be to jot down an answer from a phone call to
-# the institution represented by the account. It should show up in an account's
-# journal. If you don't want this rendered, use the comment syntax in the input
-# file, which does not get parsed and stored.
-#
-# Attributes:
-#   meta: See above.
-#   date: See above.
-#   account: A string, the account which the note is to be attached to. This is
-#     never None, notes always have an account they correspond to.
-#   comment: A free-form string, the text of the note. This can be long if you
-#     want it to.
-Note = new_directive('Note', [
-    ('account', Account),
-    ('comment', str)])
+class Pad(NamedTuple):
+    """
+    A "pad this account with this other account" directive. This directive
+    automatically inserts transactions that will make the next chronological
+    balance directive succeeds. It can be used to fill in missing date ranges of
+    transactions, as a convenience. You don't have to use this, it's sugar coating
+    in case you need it, while you're entering past history into your Ledger.
 
-# An "event value change" directive. These directives are used as string
-# variables that have different values over time. You can use these to track an
-# address, your location, your current employer, anything you like. The kind of
-# reporting that is made of these generic events is based on days and a
-# timeline. For instance, if you need to track the number of days you spend in
-# each country or state, create a "location" event and whenever you travel, add
-# an event directive to indicate its new value. You should be able to write
-# simple scripts against those in order to compute if you were present somewhere
-# for a particular number of days. Here's an illustrative example usage, in
-# order to maintain your health insurance coverage in Canada, you need to be
-# present in the country for 183 days or more, excluding trips of less than 30
-# days. There is a similar test to be done in the US by aliens to figure out if
-# they need to be considered as residents for tax purposes (the so-called
-# "substantial presence test"). By integrating these directives into your
-# bookkeeping, you can easily have a little program that computes the tests for
-# you. This is, of course, entirely optional and somewhat auxiliary to the main
-# purpose of double-entry bookkeeping, but correlates strongly with the
-# transactions you insert in it, and so it's a really convenient thing to have
-# in the same input file.
-#
-# Attributes:
-#   meta: See above.
-#   date: See above.
-#   "type": A short string, typically a single lowercase word, that defines a
-#     unique variable whose value changes over time. For example, 'location'.
-#   description: A free-form string, the value of the variable as of the date
-#     of the transaction.
-Event = new_directive('Event', [
-    ('type', str),
-    ('description', str)])
-
-# A named query declaration. This directive is used to create pre-canned queries
-# that can then be automatically run or made available to the shell, or perhaps be
-# rendered as part of a web interface. The purpose of this routine is to define
-# useful queries for the context of the particular given Beancount input file.
-#
-# Attributes:
-#   meta: See above.
-#   date: The date at which this query should be run. All directives following
-#     this date will be ignored automatically. This is essentially equivalent to
-#     the CLOSE modifier in the shell syntax.
-#   name: A string, the unique identifier for the query.
-#   query_string: The SQL query string to be run or made available.
-Query = new_directive('Query', [
-    ('name', str),
-    ('query_string', str)])
-
-# A price declaration directive. This establishes the price of a currency in
-# terms of another currency as of the directive's date. A history of the prices
-# for each currency pairs is built and can be queried within the bookkeeping
-# system. Note that because Beancount does not store any data at time-of-day
-# resolution, it makes no sense to have multiple price directives at the same
-# date. (Beancount will not attempt to solve this problem; this is beyond the
-# general scope of double-entry bookkeeping and if you need to build a day
-# trading system, you should probably use something else).
-#
-# Attributes:
-#   meta: See above.
-#   date: See above.
-#  currency: A string, the currency that is being priced, e.g. HOOL.
-#  amount: An instance of Amount, the number of units and currency that
-#    'currency' is worth, for instance 1200.12 USD.
-Price = new_directive('Price', [
-    ('currency', Currency),
-    ('amount', Amount)])
-
-# A document file declaration directive. This directive is used to attach a
-# statement to an account, at a particular date. A typical usage would be to
-# render PDF files or scans of your bank statements into the account's journal.
-# While you can explicitly create those directives in the input syntax, it is
-# much more convenient to provide Beancount with a root directory to search for
-# filenames in a hierarchy mirroring the chart of accounts, filenames which
-# should match the following dated format: "YYYY-MM-DD.*". See options for
-# detail. Beancount will automatically create these documents directives based
-# on the file hierarchy, and you can get them by parsing the list of entries.
-#
-# Attributes:
-#   meta: See above.
-#   date: See above.
-#   account: A string, the account which the statement or document is associated
-#     with.
-#   filename: The absolute filename of the document file.
-#   tags: A set of tag strings (without the '#'), or None, if an empty set.
-#   links: A set of link strings (without the '^'), or None, if an empty set.
-Document = new_directive('Document', [
-    ('account', Account),
-    ('filename', str),
-    ('tags', Optional[Set]),
-    ('links', Optional[Set])])
+    Attributes:
+      meta: See above.
+      date: See above.
+      account: A string, the name of the account which needs to be filled.
+      source_account: A string, the name of the account which is used to debit from
+        in order to fill 'account'.
+    """
+    meta: Meta
+    date: datetime.date
+    account: Account
+    source_account: Account
 
 
-# A custom directive. This directive can be used to implement new experimental
-# dated features in the Beancount file. This is meant as an intermediate measure
-# to be used when you would need to implement a new directive in a plugin. These
-# directives will be parsed liberally... any list of tokens are supported. All
-# that is required is some unique name for them that acts as a "type". These
-# directives are included in the stream and a plugin should be able to gather
-# them.
-#
-# Attributes:
-#   meta: See above.
-#   date: The date at which this query should be run. All directives following
-#     this date will be ignored automatically. This is essentially equivalent to
-#     the CLOSE modifier in the shell syntax.
-#   dir_type: A string that represents the type of the directive.
-#   values: A list of values of various simple types supported by the grammar.
-#     (Note that this list is not enforced to be consistent for all directives
-#     of the same type by the parser.)
-Custom = new_directive('Custom', [
-    ('type', str),
-    ('values', List)])
+class Balance(NamedTuple):
+    """
+    A "check the balance of this account" directive. This directive asserts that
+    the declared account should have a known number of units of a particular
+    currency at the beginning of its date. This is essentially an assertion, and
+    corresponds to the final "Statement Balance" line of a real-world statement.
+    These assertions act as checkpoints to help ensure that you have entered your
+    transactions correctly.
+
+    Attributes:
+      meta: See above.
+      date: See above.
+      account: A string, the account whose balance to check at the given date.
+      amount: An Amount, the number of units of the given currency you're
+        expecting 'account' to have at this date.
+      diff_amount: None if the balance check succeeds. This value is set to
+        an Amount instance if the balance fails, the amount of the difference.
+      tolerance: A Decimal object, the amount of tolerance to use in the
+        verification.
+    """
+    meta: Meta
+    date: datetime.date
+    account: Account
+    amount: Amount
+    tolerance: Optional[Decimal]
+    diff_amount: Optional[Amount]
+
+
+class Posting(NamedTuple):
+    """
+    Postings are contained in Transaction entries. These represent the individual
+    legs of a transaction. Note: a posting may only appear within a single entry
+    (multiple transactions may not share a Posting instance), and that's what the
+    entry field should be set to.
+
+    Attributes:
+      account: A string, the account that is modified by this posting.
+      units: An Amount, the units of the position, or None if it is to be
+        inferred from the other postings in the transaction.
+      cost: A Cost or CostSpec instances, the units of the position.
+      price: An Amount, the price at which the position took place, or
+        None, where not relevant. Providing a price member to a posting
+        automatically adds a price in the prices database at the date of the
+        transaction.
+      flag: An optional flag, a one-character string or None, which is to be
+        associated with the posting. Most postings don't have a flag, but it can
+        be convenient to mark a particular posting as problematic or pending to
+        be reconciled for a future import of its account.
+      meta: A dict of strings to values, the metadata that was attached
+        specifically to that posting, or None, if not provided. In practice, most
+        of the instances will be unlikely to have metadata.
+    """
+    account: Account
+    units: Optional[Amount]
+    cost: Optional[Union[Cost, CostSpec]]
+    price: Optional[Amount]
+    flag: Optional[Flag]
+    meta: Optional[Meta]
+
+
+class Transaction(NamedTuple):
+    """
+    A transaction! This is the main type of object that we manipulate, and the
+    entire reason this whole project exists in the first place, because
+    representing these types of structures with a spreadsheet is difficult.
+
+    Attributes:
+      meta: See above.
+      date: See above.
+      flag: A single-character string or None. This user-specified string
+        represents some custom/user-defined state of the transaction. You can use
+        this for various purposes. Otherwise common, pre-defined flags are defined
+        under beancount.core.flags, to flags transactions that are automatically
+        generated.
+      payee: A free-form string that identifies the payee, or None, if absent.
+      narration: A free-form string that provides a description for the transaction.
+        All transactions have at least a narration string, this is never None.
+      tags: A set of tag strings (without the '#'), or EMPTY_SET.
+      links: A set of link strings (without the '^'), or EMPTY_SET.
+      postings: A list of Posting instances, the legs of this transaction. See the
+        doc under Posting above.
+    """
+    meta: Meta
+    date: datetime.date
+    flag: Flag
+    payee: Optional[str]
+    narration: str
+    tags: FrozenSet
+    links: FrozenSet
+    postings: List[Posting]
+
+
+class TxnPosting(NamedTuple):
+    """
+    A pair of a Posting and its parent Transaction. This is inserted as
+    temporaries in lists of postings-of-entries, which is the product of a
+    realization.
+
+    Attributes:
+      txn: The parent Transaction instance.
+      posting: The Posting instance.
+    """
+    txn: Transaction
+    posting: Posting
+
+
+class Note(NamedTuple):
+    """
+    A note directive, a general note that is attached to an account. These are
+    used to attach text at a particular date in a specific account. The notes can
+    be anything; a typical use would be to jot down an answer from a phone call to
+    the institution represented by the account. It should show up in an account's
+    journal. If you don't want this rendered, use the comment syntax in the input
+    file, which does not get parsed and stored.
+
+    Attributes:
+      meta: See above.
+      date: See above.
+      account: A string, the account which the note is to be attached to. This is
+        never None, notes always have an account they correspond to.
+      comment: A free-form string, the text of the note. This can be long if you
+        want it to.
+    """
+    meta: Meta
+    date: datetime.date
+    account: Account
+    comment: str
+    tags: Optional[Set]
+    links: Optional[Set]
+
+
+class Event(NamedTuple):
+    """
+    An "event value change" directive. These directives are used as string
+    variables that have different values over time. You can use these to track an
+    address, your location, your current employer, anything you like. The kind of
+    reporting that is made of these generic events is based on days and a
+    timeline. For instance, if you need to track the number of days you spend in
+    each country or state, create a "location" event and whenever you travel, add
+    an event directive to indicate its new value. You should be able to write
+    simple scripts against those in order to compute if you were present somewhere
+    for a particular number of days. Here's an illustrative example usage, in
+    order to maintain your health insurance coverage in Canada, you need to be
+    present in the country for 183 days or more, excluding trips of less than 30
+    days. There is a similar test to be done in the US by aliens to figure out if
+    they need to be considered as residents for tax purposes (the so-called
+    "substantial presence test"). By integrating these directives into your
+    bookkeeping, you can easily have a little program that computes the tests for
+    you. This is, of course, entirely optional and somewhat auxiliary to the main
+    purpose of double-entry bookkeeping, but correlates strongly with the
+    transactions you insert in it, and so it's a really convenient thing to have
+    in the same input file.
+
+    Attributes:
+      meta: See above.
+      date: See above.
+      "type": A short string, typically a single lowercase word, that defines a
+        unique variable whose value changes over time. For example, 'location'.
+      description: A free-form string, the value of the variable as of the date
+        of the transaction.
+    """
+    meta: Meta
+    date: datetime.date
+    type: str
+    description: str
+
+
+class Query(NamedTuple):
+    """
+    A named query declaration. This directive is used to create pre-canned queries
+    that can then be automatically run or made available to the shell, or perhaps be
+    rendered as part of a web interface. The purpose of this routine is to define
+    useful queries for the context of the particular given Beancount input file.
+
+    Attributes:
+      meta: See above.
+      date: The date at which this query should be run. All directives following
+        this date will be ignored automatically. This is essentially equivalent to
+        the CLOSE modifier in the shell syntax.
+      name: A string, the unique identifier for the query.
+      query_string: The SQL query string to be run or made available.
+    """
+    meta: Meta
+    date: datetime.date
+    name: str
+    query_string: str
+
+
+class Price(NamedTuple):
+    """
+    A price declaration directive. This establishes the price of a currency in
+    terms of another currency as of the directive's date. A history of the prices
+    for each currency pairs is built and can be queried within the bookkeeping
+    system. Note that because Beancount does not store any data at time-of-day
+    resolution, it makes no sense to have multiple price directives at the same
+    date. (Beancount will not attempt to solve this problem; this is beyond the
+    general scope of double-entry bookkeeping and if you need to build a day
+    trading system, you should probably use something else).
+
+    Attributes:
+      meta: See above.
+      date: See above.
+     currency: A string, the currency that is being priced, e.g. HOOL.
+     amount: An instance of Amount, the number of units and currency that
+       'currency' is worth, for instance 1200.12 USD.
+    """
+    meta: Meta
+    date: datetime.date
+    currency: Currency
+    amount: Amount
+
+
+class Document(NamedTuple):
+    """
+    A document file declaration directive. This directive is used to attach a
+    statement to an account, at a particular date. A typical usage would be to
+    render PDF files or scans of your bank statements into the account's journal.
+    While you can explicitly create those directives in the input syntax, it is
+    much more convenient to provide Beancount with a root directory to search for
+    filenames in a hierarchy mirroring the chart of accounts, filenames which
+    should match the following dated format: "YYYY-MM-DD.*". See options for
+    detail. Beancount will automatically create these documents directives based
+    on the file hierarchy, and you can get them by parsing the list of entries.
+
+    Attributes:
+      meta: See above.
+      date: See above.
+      account: A string, the account which the statement or document is associated
+        with.
+      filename: The absolute filename of the document file.
+      tags: A set of tag strings (without the '#'), or None, if an empty set.
+      links: A set of link strings (without the '^'), or None, if an empty set.
+    """
+    meta: Meta
+    date: datetime.date
+    account: Account
+    filename: str
+    tags: Optional[Set]
+    links: Optional[Set]
+
+
+class Custom(NamedTuple):
+    """
+    A custom directive. This directive can be used to implement new experimental
+    dated features in the Beancount file. This is meant as an intermediate measure
+    to be used when you would need to implement a new directive in a plugin. These
+    directives will be parsed liberally... any list of tokens are supported. All
+    that is required is some unique name for them that acts as a "type". These
+    directives are included in the stream and a plugin should be able to gather
+    them.
+
+    Attributes:
+      meta: See above.
+      type: A string that represents the type of the directive.
+      values: A list of values of various simple types supported by the grammar.
+        (Note that this list is not enforced to be consistent for all directives
+        of the same type by the parser.)
+    """
+    meta: Meta
+    date: datetime.date
+    type: str
+    values: List
 
 
 # A list of all the valid directive types.
@@ -397,8 +449,27 @@ Directive = Union[
     Custom
 ]
 
+
+class dtypes:  # pylint: disable=invalid-name
+    "Types of directives."
+    Open = Open
+    Close = Close
+    Commodity = Commodity
+    Pad = Pad
+    Balance = Balance
+    Transaction = Transaction
+    Note = Note
+    Event = Event
+    Query = Query
+    Price = Price
+    Document = Document
+    Custom = Custom
+
+
 # Type for the list of entries and options map.
+# Note: In v3, make the terminology 'Entries' obsolete.
 Entries = List[Directive]
+Directives = List[Directive]
 Options = Dict[str, Any]
 
 
