@@ -14,13 +14,17 @@ from beancount.core.amount import A
 from beancount.core import amount
 from beancount.core.position import Position
 from beancount.core.position import Cost
-from beancount.core.inventory import Inventory
-from beancount.core.inventory import Booking
 from beancount.core import convert
 from beancount.core import position
-from beancount.core import inventory
 from beancount.utils import invariants
 
+try:
+    from beancount.core import inventory
+except ImportError:
+    from beancount.ccore import _core as inventory
+Inventory = inventory.Inventory
+MatchResult = inventory.MatchResult
+
 
 P = position.from_string
 I = inventory.from_string
@@ -35,19 +39,17 @@ def tearDown(module):
     invariants.uninstrument_invariants(Inventory)
 
 
-class TestInventory(unittest.TestCase):
-
-    def checkAmount(self, inventory, number, currency):
-        if isinstance(number, str):
-            number = D(number)
-        amount_ = amount.Amount(number, currency)
-        inv_amount = inventory.get_currency_units(amount_.currency)
-        self.assertEqual(inv_amount, amount_)
+class TestInventoryNew(unittest.TestCase):
 
     def test_from_string(self):
         inv = inventory.from_string('')
         self.assertEqual(Inventory(), inv)
 
+
+
+class TestInventory(unittest.TestCase):
+
+    def test_from_string(self):
         inv = inventory.from_string('10 USD')
         self.assertEqual(
             Inventory([Position(A("10 USD"))]),
@@ -88,6 +90,13 @@ class TestInventory(unittest.TestCase):
                        Position(A("100 CAD"))]),
             inv)
 
+    def checkAmount(self, inventory, number, currency):
+        if isinstance(number, str):
+            number = D(number)
+        amount_ = amount.Amount(number, currency)
+        inv_amount = inventory.get_currency_units(amount_.currency)
+        self.assertEqual(inv_amount, amount_)
+
     def test_ctor_empty_len(self):
         # Test regular constructor.
         inv = Inventory()
@@ -274,6 +283,17 @@ class TestInventory(unittest.TestCase):
             'HOOL': I('2.2 HOOL {532.43 USD}, '
                       '2.3 HOOL {564.00 USD, 2015-07-14}')}, ccymap)
 
+    def test_split(self):
+        inv = I('2.2 HOOL {532.43 USD}, '
+                '2.3 HOOL {564.00 USD, 2015-07-14}, '
+                '3.41 CAD, 101.20 USD')
+        ccymap = inv.split()
+        self.assertEqual({
+            'CAD': I('3.41 CAD'),
+            'USD': I('101.20 USD'),
+            'HOOL': I('2.2 HOOL {532.43 USD}, '
+                      '2.3 HOOL {564.00 USD, 2015-07-14}')}, ccymap)
+
     def test_units1(self):
         inv = Inventory()
         self.assertEqual(inv.reduce(convert.get_units), I(''))
@@ -387,16 +407,16 @@ class TestInventory(unittest.TestCase):
     def test_add_amount__booking(self):
         inv = Inventory()
         _, booking = inv.add_amount(A('100.00 USD'))
-        self.assertEqual(Booking.CREATED, booking)
+        self.assertEqual(MatchResult.CREATED, booking)
 
         _, booking = inv.add_amount(A('20.00 USD'))
-        self.assertEqual(Booking.AUGMENTED, booking)
+        self.assertEqual(MatchResult.AUGMENTED, booking)
 
         _, booking = inv.add_amount(A('-20 USD'))
-        self.assertEqual(Booking.REDUCED, booking)
+        self.assertEqual(MatchResult.REDUCED, booking)
 
         _, booking = inv.add_amount(A('-100 USD'))
-        self.assertEqual(Booking.REDUCED, booking)
+        self.assertEqual(MatchResult.REDUCED, booking)
 
     def test_add_amount__multi_currency(self):
         inv = Inventory()
@@ -505,4 +525,6 @@ class TestInventory(unittest.TestCase):
 
 
 if __name__ == '__main__':
+    if not hasattr(inventory, "__copyright__"):
+        del TestInventory
     unittest.main()
