@@ -4,7 +4,9 @@ __copyright__ = "Copyright (C) 2013-2016  Martin Blais"
 __license__ = "GNU GPLv2"
 
 from os import path
+from typing import Any, Optional
 import collections
+import copy
 import functools
 import glob
 import hashlib
@@ -15,11 +17,11 @@ import logging
 import os
 import pickle
 import struct
-import traceback
+import sys
 import textwrap
 import time
+import traceback
 import warnings
-from typing import Optional
 
 from beancount.utils import misc_utils
 from beancount.core import data
@@ -32,16 +34,26 @@ from beancount.utils import encryption
 from beancount.utils import file_utils
 
 
+OptionsMap = Any
+
+
 LoadError = collections.namedtuple('LoadError', 'source message entry')
 
 
 # List of default plugins to run.
-DEFAULT_PLUGINS_PRE = [
-    ("beancount.ops.pad", None),
+PLUGINS_PRE = [
     ("beancount.ops.documents", None),
     ]
 
-DEFAULT_PLUGINS_POST = [
+# List of plugins that get enabled by --auto, and list of plugins actually
+# inserted in the list. See {4ec6a3205b6c}.
+DEFAULT_PLUGINS_AUTO = [
+    ("beancount.plugins.auto", None),
+    ]
+PLUGINS_AUTO = []
+
+PLUGINS_POST = [
+    ("beancount.ops.pad", None),
     ("beancount.ops.balance", None),
     ]
 
@@ -64,14 +76,15 @@ def load_file(filename, log_timings=None, log_errors=None, extra_validations=Non
     Args:
       filename: The name of the file to be parsed.
       log_timings: A file object or function to write timings to,
-        or None, if it should remain quiet.
+        or None, if it should remain quiet. (Note that this is intended to use
+        the logging methods and does not insert a newline.)
       log_errors: A file object or function to write errors to,
         or None, if it should remain quiet.
       extra_validations: A list of extra validation functions to run after loading
         this list of entries.
       encoding: A string or None, the encoding to decode the input filename with.
     Returns:
-      A triple of (entries, errors, options_map) where "entries" is a date-sorted
+      A triple of (entries, errors, option_map) where "entries" is a date-sorted
       list of entries from the file, "errors" a list of error objects generated
       while parsing and validating the file, and "options_map", a dict of the
       options parsed from the file.
@@ -106,7 +119,7 @@ def load_encrypted_file(filename, log_timings=None, log_errors=None, extra_valid
       dedent: See load_string().
       encoding: See load_string().
     Returns:
-      A triple of (entries, errors, options_map) where "entries" is a date-sorted
+      A triple of (entries, errors, option_map) where "entries" is a date-sorted
       list of entries from the file, "errors" a list of error objects generated
       while parsing and validating the file, and "options_map", a dict of the
       options parsed from the file.
@@ -340,6 +353,7 @@ def _parse_recursive(sources, log_timings, encoding=None):
     # Current parse state.
     entries, parse_errors = [], []
     options_map = None
+    other_options_map = []
 
     # A stack of sources to be parsed.
     source_stack = list(sources)
@@ -419,7 +433,7 @@ def _parse_recursive(sources, log_timings, encoding=None):
             if is_top_level:
                 options_map = src_options_map
             else:
-                aggregate_options_map(options_map, src_options_map)
+                other_options_map.append(src_options_map)
 
             # Add includes to the list of sources to process. chdir() for glob,
             # which uses it indirectly.
@@ -449,22 +463,37 @@ def _parse_recursive(sources, log_timings, encoding=None):
     # Save the set of parsed filenames in options_map.
     options_map['include'] = sorted(filenames_seen)
 
+    options_map = aggregate_options_map(options_map, other_options_map)
+
     return entries, parse_errors, options_map
 
 
-def aggregate_options_map(options_map, src_options_map):
+def aggregate_options_map(options_map, other_options_map):
     """Aggregate some of the attributes of options map.
 
     Args:
       options_map: The target map in which we want to aggregate attributes.
         Note: This value is mutated in-place.
-      src_options_map: A source map whose values we'd like to see aggregated.
+      other_options_map: A list of other options maps, some of whose values
+        we'd like to see aggregated.
     """
-    op_currencies = options_map["operating_currency"]
-    for currency in src_options_map["operating_currency"]:
-        if currency not in op_currencies:
-            op_currencies.append(currency)
-    options_map["dcontext"].update_from(src_options_map["dcontext"])
+    options_map = copy.copy(options_map)
+
+    currencies = list(options_map["operating_currency"])
+    for omap in other_options_map:
+        currencies.extend(omap["operating_currency"])
+        options_map["dcontext"].update_from(omap["dcontext"])
+    options_map["operating_currency"] = list(misc_utils.uniquify(currencies))
+
+
+    # Produce a 'pythonpath' value for transformers.
+    pythonpath = set()
+    for omap in itertools.chain((options_map,), other_options_map):
+        if omap.get("insert_pythonpath", False):
+            pythonpath.add(path.dirname(omap["filename"]))
+    options_map["pythonpath"] = sorted(pythonpath)
+
+    return options_map
 
 
 def _load(sources, log_timings, extra_validations, encoding):
@@ -509,8 +538,16 @@ def _load(sources, log_timings, extra_validations, encoding):
 
     # Transform the entries.
     with misc_utils.log_time('run_transformations', log_timings, indent=1):
-        entries, errors = run_transformations(entries, parse_errors, options_map,
-                                              log_timings)
+
+        # Insert the user PYTHONPATH entries before processing the plugins.
+        saved_pythonpath = list(sys.path)
+        try:
+            if "pythonpath" in options_map:
+                sys.path[0:0] = options_map["pythonpath"]
+            entries, errors = run_transformations(entries, parse_errors, options_map,
+                                                  log_timings)
+        finally:
+            sys.path[:] = saved_pythonpath
 
     # Validate the list of entries.
     with misc_utils.log_time('beancount.ops.validate', log_timings, indent=1):
@@ -549,9 +586,10 @@ def run_transformations(entries, parse_errors, options_map, log_timings):
     if options_map['plugin_processing_mode'] == 'raw':
         plugins_iter = options_map["plugin"]
     elif options_map['plugin_processing_mode'] == 'default':
-        plugins_iter = itertools.chain(DEFAULT_PLUGINS_PRE,
+        plugins_iter = itertools.chain(PLUGINS_PRE,
                                        options_map["plugin"],
-                                       DEFAULT_PLUGINS_POST)
+                                       PLUGINS_AUTO,
+                                       PLUGINS_POST)
     else:
         assert "Invalid value for plugin_processing_mode: {}".format(
             options_map['plugin_processing_mode'])
