@@ -14,7 +14,6 @@ from os import path
 from beancount import loader
 from beancount.parser import parser
 from beancount.utils import test_utils
-from beancount.utils import encryption_test
 
 
 TEST_INPUT = """
@@ -362,36 +361,6 @@ class TestLoadIncludes(unittest.TestCase):
                          list(map(path.basename, options_map['include'])))
 
 
-class TestLoadIncludesEncrypted(encryption_test.TestEncryptedBase):
-
-    def test_include_encrypted(self):
-        with test_utils.tempdir() as tmpdir:
-            test_utils.create_temporary_files(tmpdir, {
-                'apples.beancount': """
-                  include "oranges.beancount.asc"
-                  2014-01-01 open Assets:Apples
-                """,
-                'oranges.beancount': """
-                  2014-01-02 open Assets:Oranges
-                """})
-
-            # Encrypt the oranges file and remove the unencrypted file.
-            with open(path.join(tmpdir, 'oranges.beancount')) as infile:
-                self.encrypt_as_file(infile.read(),
-                                     path.join(tmpdir, 'oranges.beancount.asc'))
-            os.remove(path.join(tmpdir, 'oranges.beancount'))
-
-            # Load the top-level file which includes the encrypted file.
-            with test_utils.environ('GNUPGHOME', self.ringdir):
-                entries, errors, options_map = loader.load_file(
-                    path.join(tmpdir, 'apples.beancount'))
-
-        self.assertFalse(errors)
-        self.assertEqual(2, len(entries))
-        self.assertRegex(entries[0].meta['filename'], 'apples.beancount')
-        self.assertRegex(entries[1].meta['filename'], 'oranges.+count.asc')
-
-
 class TestLoadCache(unittest.TestCase):
 
     def setUp(self):
@@ -537,8 +506,8 @@ class TestLoadCache(unittest.TestCase):
     def test_load_cache_disable(self):
         with test_utils.tempdir() as tmp:
             cache_filename = path.join(tmp, "__{filename}__")
-            for kwargs in [dict(use_cache=False),
-                           dict(use_cache=False, cache_filename=cache_filename)]:
+            for kwargs in [{'use_cache': False},
+                           {'use_cache': False, 'cache_filename': cache_filename}]:
                 loader.initialize(**kwargs)
                 test_utils.create_temporary_files(tmp, {
                     'apples.beancount': """
@@ -549,25 +518,6 @@ class TestLoadCache(unittest.TestCase):
                 self.assertEqual({'apples.beancount'}, set(os.listdir(tmp)))
 
 
-class TestEncoding(unittest.TestCase):
-
-    def test_string_unicode(self):
-        utf8_bytes = textwrap.dedent("""
-          2015-01-01 open Assets:Something
-          2015-05-23 note Assets:Something "¡¢£¤¥¦§¨©ª«¬®¯°±²³´µ¶·¸¹º»¼ "
-        """).encode('utf-8')
-        entries, errors, options_map = loader.load_string(utf8_bytes, encoding='utf8')
-        self.assertFalse(errors)
-
-    def test_string_latin1(self):
-        utf8_bytes = textwrap.dedent("""
-          2015-01-01 open Assets:Something
-          2015-05-23 note Assets:Something "¡¢£¤¥¦§¨©ª«¬®¯°±²³´µ¶·¸¹º»¼ "
-        """).encode('latin1')
-        entries, errors, options_map = loader.load_string(utf8_bytes, encoding='latin1')
-        self.assertFalse(errors)
-
-
 class TestOptionsAggregation(unittest.TestCase):
 
     def test_aggregate_operating_currencies(self):
