@@ -111,8 +111,16 @@ def render_entry_context(entries, options_map, entry, parsed_entry=None):
     pr(header.format("Balances before transaction"))
     pr()
     before_hashes = set()
+    average_costs = {}
     for account in accounts:
-        positions = balance_before[account].get_positions()
+        balance = balance_before[account]
+
+        pc_balances = balance.split()
+        for currency, pc_balance in pc_balances.items():
+            if len(pc_balance) > 1:
+                average_costs[account] = pc_balance.average()
+
+        positions = balance.get_positions()
         for position in positions:
             before_hashes.add((account, hash(position)))
             pr(position_line.format('', account, str(position)))
@@ -121,6 +129,16 @@ def render_entry_context(entries, options_map, entry, parsed_entry=None):
         pr()
     pr()
 
+    # Print average cost per account, if relevant.
+    if average_costs:
+        pr(header.format("Average Costs"))
+        pr()
+        for account, average_cost in sorted(average_costs.items()):
+            for position in average_cost:
+                pr(position_line.format('', account, str(position)))
+        pr()
+        pr()
+
     # Print the entry itself.
     dcontext = options_map['dcontext']
     pr(header.format("Unbooked Transaction"))
