@@ -5,13 +5,13 @@ __license__ = "GNU GPLv2"
 
 import collections
 import copy
-import re
-import sys
 import traceback
 from os import path
 from datetime import date
 from decimal import Decimal
 
+import regex
+
 from beancount.core.number import ZERO
 from beancount.core.number import MISSING
 from beancount.core.amount import Amount
@@ -92,9 +92,9 @@ def valid_account_regexp(options):
     # Replace the first term of the account regular expression with the specific
     # names allowed under the options configuration. This code is kept in sync
     # with {5672c7270e1e}.
-    return re.compile("(?:{})(?:{}{})+".format('|'.join(names),
-                                               account.sep,
-                                               account.ACC_COMP_NAME_RE))
+    return regex.compile("(?:{})(?:{}{})+".format('|'.join(names),
+                                                  account.sep,
+                                                  account.ACC_COMP_NAME_RE))
 
 
 # A temporary data structure used during parsing to hold and accumulate the
@@ -386,10 +386,6 @@ class Builder(lexer.LexBuilder):
             if key.startswith('name_'):
                 # Update the set of valid account types.
                 self.account_regexp = valid_account_regexp(self.options)
-            elif key == 'insert_pythonpath':
-                # Insert the PYTHONPATH to this file when and only if you
-                # encounter this option.
-                sys.path.insert(0, path.dirname(filename))
 
     def include(self, filename, lineno, include_filename):
         """Process an include directive.
@@ -704,7 +700,7 @@ class Builder(lexer.LexBuilder):
         meta = new_metadata(filename, lineno, kvlist)
         return Price(meta, date, currency, amount)
 
-    def note(self, filename, lineno, date, account, comment, kvlist):
+    def note(self, filename, lineno, date, account, comment, tags_links, kvlist):
         """Process a note directive.
 
         Args:
@@ -718,7 +714,8 @@ class Builder(lexer.LexBuilder):
           A new Note object.
         """
         meta = new_metadata(filename, lineno, kvlist)
-        return Note(meta, date, account, comment)
+        tags, links = self._finalize_tags_links(tags_links.tags, tags_links.links)
+        return Note(meta, date, account, comment, tags, links)
 
     def document(self, filename, lineno, date, account, document_filename, tags_links,
                  kvlist):
