@@ -109,6 +109,7 @@ corresponding expected values.
 __copyright__ = "Copyright (C) 2013-2016  Martin Blais"
 __license__ = "GNU GPLv2"
 
+import codecs
 import contextlib
 import functools
 import inspect
@@ -176,7 +177,8 @@ def is_entry_incomplete(entry):
     return False
 
 
-def parse_file(file, report_filename=None, report_firstline=1, **kw):
+def parse_file(file, report_filename=None, report_firstline=1,
+               encoding=None, debug=False, **kw):
     """Parse a beancount input file and return Ledger with the list of
     transactions and tree of accounts.
 
@@ -189,6 +191,8 @@ def parse_file(file, report_filename=None, report_firstline=1, **kw):
         list of errors that were encountered during parsing, and
         a dict of the option values that were parsed from the file.)
     """
+    if encoding is not None and codecs.lookup(encoding).name != 'utf-8':
+        raise ValueError('Only UTF-8 encoded files are supported.')
     with contextlib.ExitStack() as ctx:
         if file == '-':
             file = sys.stdin.buffer
@@ -198,12 +202,12 @@ def parse_file(file, report_filename=None, report_firstline=1, **kw):
         elif not isinstance(file, io.IOBase):
             file = ctx.enter_context(open(file, 'rb'))
         builder = grammar.Builder()
-        parser = _parser.Parser(builder)
+        parser = _parser.Parser(builder, debug=debug)
         parser.parse(file, filename=report_filename, lineno=report_firstline, **kw)
     return builder.finalize()
 
 
-def parse_string(string, report_filename=None, **kw):
+def parse_string(string, report_filename=None, dedent=False, **kw):
     """Parse a beancount input file and return Ledger with the list of
     transactions and tree of accounts.
 
@@ -212,12 +216,12 @@ def parse_string(string, report_filename=None, **kw):
       report_filename: A string, the source filename from which this string
         has been extracted, if any. This is stored in the metadata of the
         parsed entries.
-      **kw: See parse.c. This function parses out 'dedent' which removes
-        whitespace from the front of the text (default is False).
+      dedent: Whether to run textwrap.dedent() on the string before parsing.
+      **kw: See parse.c.
     Return:
       Same as the output of parse_file().
     """
-    if kw.pop('dedent', None):
+    if dedent:
         string = textwrap.dedent(string)
     if isinstance(string, str):
         string = string.encode('utf8')
