@@ -2,10 +2,12 @@ __copyright__ = "Copyright (C) 2014-2016  Martin Blais"
 __license__ = "GNU GPLv2"
 
 import io
+import os
 from datetime import date
 import unittest
 import re
 import textwrap
+import tempfile
 
 from beancount.parser import printer
 from beancount.parser import cmptest
@@ -70,6 +72,122 @@ class TestEntryPrinter(cmptest.TestCase):
         # Compare the two output texts.
         self.assertEqual(oss2.getvalue(), oss1.getvalue())
 
+    def assertRoundTripViaRealFile(self, entries1, errors1,
+                                   test_write_source=True):
+        """
+        The same as assertRoundTrip, but the 1st time saves ledger in string
+        format to a real file as an opposite to io.StringIO().
+        This ensures, that when parsed back from file, entries will contain file
+        name and line number metadata.
+
+        Args:
+            test_write_source - if True, will test funtionality write_source,
+            which activates printing of the source file and line number for
+            every transaction
+        """
+        self.assertFalse(errors1)
+
+        # Not using context manager to make this test compatible with Windows
+        # Contex manager can be used as of python 3.12
+        # see https://github.com/python/cpython/issues/58451
+        # https://github.com/beancount/beancount/issues/222
+        oss1 = tempfile.NamedTemporaryFile('w', suffix='.beancount',
+                                           delete=False)
+
+        oss1.write('option "plugin_processing_mode" "raw"\n')
+        printer.print_entries(entries1, file=oss1)
+
+        oss1.close()
+
+        with open(oss1.name, "r") as file:
+            oss1_value = file.read()
+
+        # entries2 will contain information about filename and line number in
+        # metadata
+        entries2, errors, __ = loader.load_file(oss1.name)
+
+        self.assertEqualEntries(entries1, entries2)
+        self.assertFalse(errors)
+
+        oss2 = io.StringIO()
+        oss2.write('option "plugin_processing_mode" "raw"\n')
+
+        # converting entries2 to string now also with the commented out source
+        # information
+        printer.print_entries(entries2, file=oss2, write_source=test_write_source)
+        oss2_value = oss2.getvalue()
+        entries3, errors, __ = loader.load_string(oss2_value)
+
+        # testing, that qnt of '; source' subscrings  is equal to qnt of
+        # entries
+        if test_write_source:
+            self.assertEqual(oss2_value.count("; source"), len(entries3))
+
+        self.assertEqualEntries(entries1, entries3)
+        self.assertFalse(errors)
+
+        if test_write_source:
+            self.assertNotEqual(oss2_value, oss1_value)
+        else:
+            self.assertEqual(oss2.getvalue(), oss1_value)
+
+        # Finally deleting temporary file
+        os.unlink(oss1.name)
+
+    def test_write_source(self):
+        """
+        Targeted testing of the write_source, functionality which activates
+        printing of the source file and line number for every transaction in
+        a commented out form
+        """
+        ORIGINAL_LEDGER = textwrap.dedent("""\
+        2014-01-01 open Assets:Account1
+        2014-01-01 open Assets:Cash
+        
+        2014-06-08 *
+          Assets:Account1       111.00 BEAN
+          Assets:Cash          -111.00 BEAN
+        """)
+
+        # note:this multistring contains some training white spaces
+        EXPECTED_OUTPUT_LEDGER_TEMPL = textwrap.dedent("""
+        ; source: __path_to_file__:1:      
+        2014-01-01 open Assets:Account1
+        
+        ; source: __path_to_file__:2:      
+        2014-01-01 open Assets:Cash
+        
+        ; source: __path_to_file__:4:      
+        2014-06-08 * 
+          Assets:Account1   111.00 BEAN
+          Assets:Cash      -111.00 BEAN
+        """)
+
+
+        # Not using context manager to make this test compatible with Windows
+        # Contex manager can be used as of python 3.12
+        # see https://github.com/python/cpython/issues/58451
+        # https://github.com/beancount/beancount/issues/222
+        oss1 = tempfile.NamedTemporaryFile('w', suffix='.beancount',
+                                           delete=False)
+        oss1.write(ORIGINAL_LEDGER)
+        oss1.close()
+        # entries2 will contain the file name and line number in metadata
+        entries2, errors, __ = loader.load_file(oss1.name)
+        self.assertFalse(errors)
+
+        expected_output_ledger = EXPECTED_OUTPUT_LEDGER_TEMPL.replace(
+            '__path_to_file__',oss1.name)
+
+        # Print out those reparsed and parse them back in.
+        oss2 = io.StringIO()
+        # Getting original ledger but with the source location information
+        printer.print_entries(entries2, file=oss2, write_source=True)
+
+        self.maxDiff=1000
+        self.assertEqual(expected_output_ledger, oss2.getvalue())
+
+
     @loader.load_doc()
     def test_Transaction(self, entries, errors, __):
         """
@@ -123,7 +241,12 @@ class TestEntryPrinter(cmptest.TestCase):
 
         2014-06-20 custom "budget" Assets:Account2 "balance < 200.00 USD" 200.00 10.00 USD
         """
-        self.assertRoundTrip(entries, errors)
+        with self.subTest("RoundTrip test via StringIO"):
+            self.assertRoundTrip(entries, errors)
+
+        #TODO: This test fails on Windows for the entry with escaped symbols
+        with self.subTest("RoundTrip test via real file"):
+            self.assertRoundTripViaRealFile(entries, errors)
 
     @loader.load_doc()
     def test_Balance(self, entries, errors, __):
@@ -131,7 +254,29 @@ class TestEntryPrinter(cmptest.TestCase):
         2014-06-01 open Assets:Account1
         2014-06-08 balance Assets:Account1     0.00 USD
         """
-        self.assertRoundTrip(entries, errors)
+        with self.subTest("RoundTrip test via StringIO"):
+            self.assertRoundTrip(entries, errors)
+
+        with self.subTest("RoundTrip test via real file"):
+            self.assertRoundTripViaRealFile(entries, errors)
+
+    @loader.load_doc()
+    def test_BalanceTolerance(self, entries, errors, __):
+        """
+        2014-06-01 open Assets:Account1
+        2014-06-01 open Assets:Cash
+
+        2014-06-02 * "Deposit"
+          Assets:Account1       199.95 USD
+          Assets:Cash          -199.95 USD
+
+        2014-06-04 balance Assets:Account1     200.00 ~0.05 USD
+        """
+        with self.subTest("RoundTrip test via StringIO"):
+            self.assertRoundTrip(entries, errors)
+
+        with self.subTest("RoundTrip test via real file"):
+            self.assertRoundTripViaRealFile(entries, errors)
 
     @loader.load_doc()
     def test_Note(self, entries, errors, __):
@@ -139,7 +284,11 @@ class TestEntryPrinter(cmptest.TestCase):
         2014-06-01 open Assets:Account1
         2014-06-08 note Assets:Account1 "Note"
         """
-        self.assertRoundTrip(entries, errors)
+        with self.subTest("RoundTrip test via StringIO"):
+            self.assertRoundTrip(entries, errors)
+
+        with self.subTest("RoundTrip test via real file"):
+            self.assertRoundTripViaRealFile(entries, errors)
 
     @loader.load_doc()
     def test_Document(self, entries, errors, __):
@@ -153,14 +302,22 @@ class TestEntryPrinter(cmptest.TestCase):
         2014-06-08 document Assets:Account1 "path/to/document2.csv" #tag1
         2014-06-08 document Assets:Account1 "path/to/document2.csv" ^link1
         """
-        self.assertRoundTrip(entries, errors)
+        with self.subTest("RoundTrip test via StringIO"):
+            self.assertRoundTrip(entries, errors)
+
+        with self.subTest("RoundTrip test via real file"):
+            self.assertRoundTripViaRealFile(entries, errors)
 
     @loader.load_doc()
     def test_Query(self, entries, errors, __):
         """
         2014-06-08 query "cash" "SELECT sum(position) WHERE currency = 'USD'"
         """
-        self.assertRoundTrip(entries, errors)
+        with self.subTest("RoundTrip test via StringIO"):
+            self.assertRoundTrip(entries, errors)
+
+        with self.subTest("RoundTrip test via real file"):
+            self.assertRoundTripViaRealFile(entries, errors)
 
     @loader.load_doc()
     def test_Pad(self, entries, errors, __):
@@ -170,7 +327,11 @@ class TestEntryPrinter(cmptest.TestCase):
         2014-06-08 pad Assets:Account1 Assets:Account2
         2014-10-01 balance Assets:Account1  1 USD
         """
-        self.assertRoundTrip(entries, errors)
+        with self.subTest("RoundTrip test via StringIO"):
+            self.assertRoundTrip(entries, errors)
+
+        with self.subTest("RoundTrip test via real file"):
+            self.assertRoundTripViaRealFile(entries, errors)
 
     @loader.load_doc()
     def test_Open(self, entries, errors, __):
@@ -180,7 +341,11 @@ class TestEntryPrinter(cmptest.TestCase):
         2014-06-08 open Assets:Account3  USD,CAD,EUR
         2014-06-08 open Assets:Account4  HOOL   "NONE"
         """
-        self.assertRoundTrip(entries, errors)
+        with self.subTest("RoundTrip test via StringIO"):
+            self.assertRoundTrip(entries, errors)
+
+        with self.subTest("RoundTrip test via real file"):
+            self.assertRoundTripViaRealFile(entries, errors)
 
     @loader.load_doc()
     def test_Close(self, entries, errors, __):
@@ -188,7 +353,11 @@ class TestEntryPrinter(cmptest.TestCase):
         2014-01-01 open  Assets:Account1
         2014-06-08 close Assets:Account1
         """
-        self.assertRoundTrip(entries, errors)
+        with self.subTest("RoundTrip test via StringIO"):
+            self.assertRoundTrip(entries, errors)
+
+        with self.subTest("RoundTrip test via real file"):
+            self.assertRoundTripViaRealFile(entries, errors)
 
     @loader.load_doc()
     def test_Price(self, entries, errors, __):
@@ -196,7 +365,11 @@ class TestEntryPrinter(cmptest.TestCase):
         2014-06-08 price  BEAN   53.24 USD
         2014-06-08 price  USD   1.09 CAD
         """
-        self.assertRoundTrip(entries, errors)
+        with self.subTest("RoundTrip test via StringIO"):
+            self.assertRoundTrip(entries, errors)
+
+        with self.subTest("RoundTrip test via real file"):
+            self.assertRoundTripViaRealFile(entries, errors)
 
     @loader.load_doc()
     def test_Event(self, entries, errors, __):
@@ -204,7 +377,11 @@ class TestEntryPrinter(cmptest.TestCase):
         2014-06-08 event "location" "New York, NY, USA"
         2014-06-08 event "employer" "Four Square"
         """
-        self.assertRoundTrip(entries, errors)
+        with self.subTest("RoundTrip test via StringIO"):
+            self.assertRoundTrip(entries, errors)
+
+        with self.subTest("RoundTrip test via real file"):
+            self.assertRoundTripViaRealFile(entries, errors)
 
     def test_metadata(self):
         meta = data.new_metadata('beancount/core/testing.beancount', 12345)
@@ -214,6 +391,7 @@ class TestEntryPrinter(cmptest.TestCase):
         self.assertEqual('  something: "a\\"\\\\c"\n', oss.getvalue())
 
 
+
 def characterize_spaces(text):
     """Classify each line to a particular type.
 
