@@ -1,12 +1,10 @@
 __copyright__ = "Copyright (C) 2018  Martin Blais"
 __license__ = "GNU GPLv2"
 
-import argparse
 import unittest
 
 import beancount
 from beancount.parser import version
-from beancount.utils import test_utils
 
 
 class TestVersion(unittest.TestCase):
@@ -30,14 +28,6 @@ class TestVersion(unittest.TestCase):
             version.compute_version_string(
                 '2.1.0', '', 0))
 
-    def test_version(self):
-        parser = version.ArgumentParser()
-        self.assertIsInstance(parser, argparse.ArgumentParser)
-
-        with test_utils.capture('stdout') as stdout, self.assertRaises(SystemExit):
-            parser.parse_args(['--version'])
-        self.assertRegex(stdout.getvalue(), r'Beancount \d+\.\d+\.\d+')
-
 
 if __name__ == '__main__':
     unittest.main()
