@@ -1,12 +1,11 @@
-"""A plugin of plugins which triggers are all the pedantic plugins.
-
-In a sense, this is the inverse of "pedantic." This is useful when doing some
-types of quick and dirty tests.
+"""A plugin of plugins which triggers all the pedantic plugins.
 """
 __copyright__ = "Copyright (C) 2017  Martin Blais"
 __license__ = "GNU GPLv2"
 
+from beancount import loader
 from beancount.plugins import check_commodity
+from beancount.plugins import check_drained
 from beancount.plugins import coherent_cost
 from beancount.plugins import leafonly
 from beancount.plugins import noduplicates
@@ -14,10 +13,7 @@ from beancount.plugins import nounused
 from beancount.plugins import onecommodity
 from beancount.plugins import sellgains
 from beancount.plugins import unique_prices
-from beancount import loader
 
-# TODO(blais): The default pedantic set of plugins should include
-# 'check_drained'. In v3 it will.
 __plugins__ = loader.combine_plugins(
     check_commodity,
     coherent_cost,
@@ -26,4 +22,6 @@ __plugins__ = loader.combine_plugins(
     nounused,
     onecommodity,
     sellgains,
-    unique_prices)
+    unique_prices,
+    check_drained,
+)
