@@ -5,16 +5,17 @@ import os
 import re
 import textwrap
 import tempfile
-from os import path
 import unittest
 
+from os import path
+
 from beancount.parser import cmptest
 from beancount.utils import test_utils
-from beancount.scripts import doctor
+from beancount.scripts.doctor import doctor
 from beancount.scripts import directories_test
 
 
-class TestScriptDoctor(test_utils.TestCase):
+class TestScriptDoctor(test_utils.ClickTestCase):
 
     @test_utils.docfile
     def test_dump_lexer(self, filename):
@@ -26,8 +27,7 @@ class TestScriptDoctor(test_utils.TestCase):
           Expenses:Restaurant   50.02 USD
           Assets:Cash
         """
-        with test_utils.capture() as stdout:
-            test_utils.run_with_args(doctor.main, ['dump-lexer', filename])
+        result = self.run_with_args(doctor, 'dump-lexer', filename)
 
         expected_output = """
             EOL               2 b'\\n'
@@ -53,14 +53,14 @@ class TestScriptDoctor(test_utils.TestCase):
             ACCOUNT           7 b'Assets:Cash'
             EOL               8 b'\\n'
         """
-        self.assertLines(expected_output, stdout.getvalue())
+        self.assertLines(expected_output, result.stdout)
 
     # pylint: disable=empty-docstring
     @test_utils.docfile
     def test_dump_lexer_empty(self, filename):
         ""
-        with test_utils.capture():
-            test_utils.run_with_args(doctor.main, ['dump-lexer', filename])
+        rv = self.run_with_args(doctor, 'dump-lexer', filename)
+        self.assertEqual(rv.stdout, "")
 
     @test_utils.docfile
     def test_dump_roundtrip(self, filename):
@@ -72,21 +72,14 @@ class TestScriptDoctor(test_utils.TestCase):
           Expenses:Restaurant   50.02 USD
           Assets:Cash
         """
-        with test_utils.capture('stdout', 'stderr'):
-            test_utils.run_with_args(doctor.main, ['roundtrip', filename])
+        self.run_with_args(doctor, 'roundtrip', filename)
 
     def test_list_options(self):
-        with test_utils.capture():
-            test_utils.run_with_args(doctor.main, ['list_options'])
-            test_utils.run_with_args(doctor.main, ['list-options'])
-
-    def test_deps(self):
-        with test_utils.capture():
-            test_utils.run_with_args(doctor.main, ['deps'])
-            test_utils.run_with_args(doctor.main, ['checkdeps'])
+        self.run_with_args(doctor, 'list_options')
 
 
-class TestScriptCheckDirectories(directories_test.TestScriptCheckDirectories):
+class TestScriptCheckDirectories(directories_test.TestScriptCheckDirectories,
+                                 test_utils.ClickTestCase):
 
     @test_utils.docfile
     def test_invocation(self, filename):
@@ -105,12 +98,12 @@ class TestScriptCheckDirectories(directories_test.TestScriptCheckDirectories):
         for directory in self.TEST_DIRECTORIES:
             os.makedirs(path.join(self.tmpdir, directory))
 
-        with test_utils.capture() as stdout:
-            test_utils.run_with_args(doctor.main, ['directories', filename, self.tmpdir])
-        self.assertEqual(2, len(stdout.getvalue().splitlines()))
-        matches = set(match.group(1) for match in re.finditer("'(.*?)'", stdout.getvalue()))
-        clean_matches = set(match[len(self.tmpdir)+1:]
-                            if match.startswith(self.tmpdir)
+        rv = self.run_with_args(doctor, 'directories', filename, self.tmpdir)
+        self.assertEqual(2, len(rv.stdout.splitlines()))
+        matches = set(match.group(1) for match in re.finditer("'(.*?)'", rv.stdout))
+        tmpdir = path.realpath(self.tmpdir)
+        clean_matches = set(match[len(tmpdir)+1:]
+                            if match.startswith(tmpdir)
                             else match
                             for match in matches)
         self.assertEqual({'Expenses/Restaurant/Sub',
@@ -119,7 +112,7 @@ class TestScriptCheckDirectories(directories_test.TestScriptCheckDirectories):
                           'Assets/Extra'}, clean_matches)
 
 
-class TestScriptMissingOpen(cmptest.TestCase):
+class TestScriptMissingOpen(cmptest.TestCase, test_utils.ClickTestCase):
 
     @test_utils.docfile
     def test_missing_open(self, filename):
@@ -137,18 +130,17 @@ class TestScriptMissingOpen(cmptest.TestCase):
               Expenses:Movie        25.00 USD
               Assets:Cash
         """
-        with test_utils.capture() as stdout:
-            test_utils.run_with_args(doctor.main, ['missing-open', filename])
+        rv = self.run_with_args(doctor, 'missing-open', filename)
 
         self.assertEqualEntries("""
 
             2014-03-03 open Expenses:Restaurant
             2014-04-04 open Expenses:Alcohol
 
-        """, stdout.getvalue())
+        """, rv.stdout)
 
 
-class TestScriptDisplayContext(cmptest.TestCase):
+class TestScriptDisplayContext(cmptest.TestCase, test_utils.ClickTestCase):
 
     @test_utils.docfile
     def test_display_context(self, filename):
@@ -166,13 +158,12 @@ class TestScriptDisplayContext(cmptest.TestCase):
               Expenses:Movie        25.00 USD
               Assets:Cash
         """
-        with test_utils.capture() as stdout:
-            test_utils.run_with_args(doctor.main, ['display-context', filename])
+        rv = self.run_with_args(doctor, 'display-context', filename)
         # Note: This probably deserves a little more love.
-        self.assertTrue(stdout.getvalue())
+        self.assertTrue(rv.stdout)
 
 
-class TestContext(cmptest.TestCase):
+class TestContext(cmptest.TestCase, test_utils.ClickTestCase):
 
     @test_utils.docfile
     def test_context(self, filename):
@@ -190,10 +181,9 @@ class TestContext(cmptest.TestCase):
               Expenses:Movie        25.00 USD
               Assets:Cash
         """
-        with test_utils.capture() as stdout:
-            test_utils.run_with_args(doctor.main, ['context', filename, '6'])
-        self.assertRegex(stdout.getvalue(), 'Location:')
-        self.assertRegex(stdout.getvalue(), '50.02')
+        rv = self.run_with_args(doctor, 'context', filename, '6')
+        self.assertRegex(rv.stdout, 'Location:')
+        self.assertRegex(rv.stdout, '50.02')
 
     @test_utils.docfile
     def test_context_multiple_files(self, filename):
@@ -217,13 +207,13 @@ class TestContext(cmptest.TestCase):
                 include "{}"
             """.format(filename)))
             topfile.flush()
-            with test_utils.capture() as stdout:
-                test_utils.run_with_args(doctor.main, ['context', topfile.name,
-                                                       '{}:6'.format(filename)])
-            self.assertRegex(stdout.getvalue(), 'Location:')
-            self.assertRegex(stdout.getvalue(), '50.02')
+            rv = self.run_with_args(doctor, 'context',
+                                   topfile.name, '{}:6'.format(filename))
+            self.assertRegex(rv.stdout, 'Location:')
+            self.assertRegex(rv.stdout, '50.02')
 
-class TestLinked(cmptest.TestCase):
+
+class TestLinked(cmptest.TestCase, test_utils.ClickTestCase):
 
     test_string = """
         2013-01-01 open Expenses:Movie
@@ -247,12 +237,11 @@ class TestLinked(cmptest.TestCase):
 
     @test_utils.docfile_extra(contents=test_string)
     def test_linked_lineno_only(self, filename):
-        with test_utils.capture() as stdout:
-            test_utils.run_with_args(doctor.main, ['linked', filename, '6'])
-        self.assertRegex(stdout.getvalue(), 'Apples')
-        self.assertRegex(stdout.getvalue(), 'Oranges')
+        rv = self.run_with_args(doctor, 'linked', filename, '6')
+        self.assertRegex(rv.stdout, 'Apples')
+        self.assertRegex(rv.stdout, 'Oranges')
         self.assertEqual(2, len(list(re.finditer(r'/(tmp|var/folders)/.*:\d+:',
-                                                 stdout.getvalue()))))
+                                                 rv.stdout))))
 
     @test_utils.docfile_extra(contents=test_string)
     def test_linked_multiple_files(self, filename):
@@ -261,22 +250,30 @@ class TestLinked(cmptest.TestCase):
                 include "{}"
             """.format(filename)))
             topfile.flush()
-            with test_utils.capture() as stdout:
-                test_utils.run_with_args(doctor.main, ['linked', topfile.name,
-                                                       '{}:6'.format(filename)])
-            self.assertRegex(stdout.getvalue(), 'Apples')
-            self.assertRegex(stdout.getvalue(), 'Oranges')
+            rv = self.run_with_args(doctor, 'linked',
+                                   topfile.name, '{}:6'.format(filename))
+            self.assertRegex(rv.stdout, 'Apples')
+            self.assertRegex(rv.stdout, 'Oranges')
             self.assertEqual(2, len(list(re.finditer(r'/(tmp|var/folders)/.*:\d+:',
-                                                     stdout.getvalue()))))
+                                                     rv.stdout))))
 
     @test_utils.docfile_extra(contents=test_string)
     def test_linked_explicit_link(self, filename):
-        with test_utils.capture() as stdout:
-            test_utils.run_with_args(doctor.main, ['linked', filename, '^abc'])
-        self.assertRegex(stdout.getvalue(), 'Apples')
-        self.assertRegex(stdout.getvalue(), 'Oranges')
+        rv = self.run_with_args(doctor, 'linked', filename, '^abc')
+        self.assertRegex(rv.stdout, 'Apples')
+        self.assertRegex(rv.stdout, 'Oranges')
         self.assertEqual(2, len(list(re.finditer(r'/(tmp|var/folders)/.*:\d+:',
-                                                 stdout.getvalue()))))
+                                                 rv.stdout))))
+
+
+class TestRegion(cmptest.TestCase, test_utils.ClickTestCase):
+
+    test_string = TestLinked.test_string
+
+    @test_utils.docfile_extra(contents=test_string)
+    def test_region(self, filename):
+        rv = self.run_with_args(doctor, 'region', filename, '4:12')
+        self.assertRegex(rv.stdout, r'Cash\s+-110.32 USD')
 
 
 if __name__ == '__main__':
